@@ -78,7 +78,7 @@ Controls the metadata options below.
 
 =item strip_metadata
 
-Not implemented yet.
+If true, any metadata in the input document is removed from the output document (note - does not take effect in complete document format).
 
 =item empty element suffix
 
@@ -102,6 +102,10 @@ The title of the generated bibliography, defaults to 'Bibliography'.
 
 Controls indent width in the generated markup, defaults to 4
 
+=item markdown_in_html_blocks
+
+Controls if Markdown is processed when inside HTML blocks. Defaults to 0.
+
 =back
 
 A number of possible items of metadata can also be supplied as options. 
@@ -167,7 +171,7 @@ use Encode      qw();
 use Carp        qw(croak);
 use base        'Exporter';
 
-our $VERSION   = '1.0.7';
+our $VERSION   = '1.0.8';
 our @EXPORT_OK = qw(markdown);
 
 ## Disabled; causes problems under Perl 5.6.1:
@@ -242,6 +246,9 @@ sub new {
     # NOTE: You can use \WikiWord to prevent a WikiWord from being treated as a link
     $p{use_wikilinks} = $p{use_wikilinks} ? 1 : 0;
     
+    # Is markdown processed in HTML blocks? See t/15inlinehtmldonotturnoffmarkdown.t
+    $p{markdown_in_html_blocks} = $p{markdown_in_html_blocks} ? 1 : 0;
+    
     $p{heading_ids} = defined $p{heading_ids} ? $p{heading_ids} : 1;
     $p{img_ids}     = defined $p{img_ids}     ? $p{img_ids}     : 1;
     
@@ -281,11 +288,13 @@ sub markdown {
         
     # Localise all of these settings, so that if they're frobbed by options here (or metadata later), the change will not persist.
     # FIXME - There should be a nicer way to do this...
-    local $self->{use_wikilinks}        = exists $options->{use_wikilinks}        ? $options->{use_wikilinks}        : $self->{use_wikilinks};
-    local $self->{empty_element_suffix} = exists $options->{empty_element_suffix} ? $options->{empty_element_suffix} : $self->{empty_element_suffix};
-    local $self->{document_format}      = exists $options->{document_format}      ? $options->{document_format}      : $self->{document_format};
-    local $self->{use_metadata}         = exists $options->{use_metadata}         ? $options->{use_metadata}         : $self->{use_metadata};
-    
+    local $self->{use_wikilinks}            = exists $options->{use_wikilinks}          ? $options->{use_wikilinks}          : $self->{use_wikilinks};
+    local $self->{empty_element_suffix}     = exists $options->{empty_element_suffix}   ? $options->{empty_element_suffix}   : $self->{empty_element_suffix};
+    local $self->{document_format}          = exists $options->{document_format}        ? $options->{document_format}        : $self->{document_format};
+    local $self->{use_metadata}             = exists $options->{use_metadata}           ? $options->{use_metadata}           : $self->{use_metadata};
+    local $self->{strip_metadata}           = exists $options->{strip_metadata}         ? $options->{strip_metadata}         : $self->{strip_metadata};
+    local $self->{markdown_in_html_blocks}  = exists $options->{markdown_in_html_blocks}? $options->{markdown_in_html_blocks}: $self->{markdown_in_html_blocks};
+
     if (exists $options->{tab_width}) {
         local $self->{tab_width} = $options->{tab_width};
     }    
@@ -345,13 +354,13 @@ sub _Markdown {
     $text =~ s/^[ \t]+$//mg;
     
     # Strip out MetaData
-    $text = $self->_ParseMetaData($text) if $self->{use_metadata};
+    $text = $self->_ParseMetaData($text) if ($self->{use_metadata} || $self->{strip_metadata});
 
     # And recheck for leading blank lines
     $text =~ s/^\n+//s;
 
     # Turn block-level HTML blocks into hash entries
-    $text = $self->_HashHTMLBlocks($text);
+    $text = $self->_HashHTMLBlocks($text) unless $self->{markdown_in_html_blocks};
 
     # Strip link definitions, store in hashes.
     $text = $self->_StripFootnoteDefinitions($text);
@@ -595,7 +604,6 @@ sub _HashHTMLBlocks {
 				"\n\n" . $key . "\n\n";
 			}egx;
 
-
 	return $text;
 }
 
@@ -767,7 +775,6 @@ sub _DoAnchors {
         my $result;
         my $whole_match = $1;
         my $link_text   = $2;
-        #warn("Got match $1 text $2");
         my $link_id     = lc $3;
 
         if ($link_id eq "") {
@@ -941,7 +948,7 @@ sub _DoImages {
             
             $label = $self->{img_ids} ? qq{ id="$label"} : '';
             $result = qq{<img$label src="$url" alt="$alt_text"};
-            if (length $link_id && defined $self->{_titles}{$link_id}) {
+            if (length $link_id && defined $self->{_titles}{$link_id} && length $self->{_titles}{$link_id}) {
                 my $title = $self->{_titles}{$link_id};
                 $title =~ s! \* !$g_escape_table{'*'}!gx;
                 $title =~ s!  _ !$g_escape_table{'_'}!gx;
@@ -1002,7 +1009,7 @@ sub _DoImages {
 
         $label = $self->{img_ids} ? qq{ id="$label"} : '';
         $result = qq{<img$label src="$url" alt="$alt_text"};
-        if (defined $title) {
+        if (defined $title && length $title) {
             $title =~ s! \* !$g_escape_table{'*'}!gx;
             $title =~ s!  _ !$g_escape_table{'_'}!gx;
             $result .=  qq{ title="$title"};
@@ -1488,6 +1495,18 @@ sub _EncodeAmpsAndAngles {
 #   $text =~ s{>(?![a-z/?\$!])}{&gt;}gi;
 #   Causes problems...
 
+    # Remove encoding inside comments
+    $text =~ s{
+        (?<=<!--) # Begin comment
+        (.*?)     # Anything inside
+        (?=-->)   # End comments
+    }{
+        my $t = $1;
+        $t =~ s/&amp;/&/g;
+        $t =~ s/&lt;/</g;
+        $t;
+    }egsx;
+
     return $text;
 }
 
@@ -1669,6 +1688,7 @@ sub _Detab {
 #
     my ($self, $text) = @_;
 
+    # FIXME - Anchor
     $text =~ s{(.*?)\t}{$1.(' ' x ($self->{tab_width} - length($1) % $self->{tab_width}))}ge;
     return $text;
 }
@@ -1686,6 +1706,7 @@ sub _ParseMetaData { # FIXME - This is really really ugly!
     foreach my $line ( split /\n/, $text ) {
         $line =~ /^\s*$/ and $inMetaData = 0 and $clean_text .= $line and next;
         if ($inMetaData) {
+            next unless $self->{use_metadata}; # We can come in here as use_metadata => 0, strip_metadata => 1
             if ($line =~ /^([a-zA-Z0-9][0-9a-zA-Z _-]+?):\s*(.*)$/ ) {
                 $currentKey = $1;
                 $currentKey =~ s/  / /g;
@@ -1749,6 +1770,8 @@ sub _StripFootnoteDefinitions {
 sub _DoFootnotes {
     my ($self, $text) = @_;
     
+    return '' unless length $text;
+    
     # First, run routines that get skipped in footnotes
     foreach my $label (sort keys %{ $self->{_footnotes} }) {
         my $footnote = $self->_RunBlockGamut($self->{_footnotes}{$label});
@@ -1769,7 +1792,6 @@ sub _DoFootnotes {
         my $id = $self->Header2Label($1);
         
         if (defined $self->{_footnotes}{$id} ) {
-#           $result = "<footnote>$self->{_footnotes}{$self->Header2Label($id)}</footnote>"
             $footnote_counter++;
             $result = qq{<a href="#$id" id="f$id" class="footnote">$footnote_counter</a>};
             push (@{ $self->{_used_footnotes} }, $id);
@@ -1843,9 +1865,11 @@ sub xhtmlMetaData {
     foreach my $key (sort keys %{$self->{_metadata}} ) {
         if (lc($key) eq "title") {
             $result.= "\t\t<title>$self->{_metadata}{$key}</title>\n";
-        } elsif (lc($key) eq "css") {
+        } 
+        elsif (lc($key) eq "css") {
             $result.= qq[\t\t<link type="text/css" rel="stylesheet" href="$self->{_metadata}{$key}"$self->{empty_element_suffix}\n];
-        } else {
+        } 
+        else {
             $result.= qq[\t\t<meta name="$key" content="$self->{_metadata}{$key}"$self->{empty_element_suffix}\n];
         }
     }
@@ -1864,6 +1888,8 @@ sub textMetaData {
     my ($self) = @_;
     my $result = "";
     
+    return $result if $self->{strip_metadata};
+
     foreach my $key (sort keys %{$self->{_metadata}} ) {
         $result .= "$key: $self->{_metadata}{$key}\n";
     }
